/***************************************************************************
 *   Copyright (C) 2010 by Martin Burgemeister, Mathis Lindemann           *
 *                                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef UCQCONSOLE_H_
#define UCQCONSOLE_H_

#include <QTextEdit>
#include <QTextCursor>
#include <QMouseEvent>
#include <QKeyEvent>
#include <QColor>
#include <QScrollBar>
#include <QAbstractSlider>
#include <QFontMetrics>
#include <QTimer>
#include <QMenu>
#include <QDebug>
#include <QFontDialog>
#include "controller/LuaController.h"
#include "tools/UCQExecuteLuaThread.h"
/**
 * \class UCQConsole
 * \brief Extended QTextEdit for use as a Lua console.
 *
 * This QTextEdit behaves in the way of a console and the entered commands are
 * executed by Lua, using the LuaController class. The console provides a
 * history, a complete function and several hotkeys.
 *
 * This class was partly taken from the QtLua library, written by Alexandre
 * Becoulet (http://www.nongnu.org/libqtlua/).
 *
 * \author Martin Burgemeister <Martin.Burgemeister@gmx.de>,
 *         Mathis Lindemann <MathisLindemann@gmx.net>
 * \version 1.0
 * \date 16 August 2010
 *
 * \see LuaController
 */
class UCQConsole: public QTextEdit
{

Q_OBJECT

public:

    UCQConsole ( QWidget* = 0, const QString& = QString ( ">" ),
                 const QStringList& = QStringList () );
    ~UCQConsole ();

    const QString& getPrompt () const;

    void setExecuteThread ( UCQExecuteLuaThread* );
    void setPrompt  ( const QString& );
    void setHistory ( const QStringList& );

    void print ( QString );
    void displayPrompt ();

protected:
    void contextMenuEvent ( QContextMenuEvent* );

private:
    void init ();
    void deleteCompletionList ();
    void checkWritingPosition ();

    void keyPressEvent ( QKeyEvent* );
    void actionKeyEnter ();

    void actionEnd  ( bool );
    void actionHome ( bool );
    void actionHistoryUp   ();
    void actionKeyComplete ();
    void actionHistoryDown ();
    void actionHistoryFind ( int );

    /**
     * \brief Format of console.
     */
    QTextCharFormat formatNormal_;
    /**
     * \brief Promt to display at beginning of the line.
     */
    QString prompt_;
    /**
     * \brief Column of prompt.
     */
    int promptStart_;
    /**
     * \brief Column of first userinput.
     */
    int lineStart_;
    /**
     * \brief Column of mark.
     */
    int mark_;
    /**
     * \brief History of commands.
     */
    QStringList history_;
    /**
     * \brief Number of commands in history.
     */
    int historyCount_;
    /**
     * \brief Number of maximum commands in history.
     */
    int historySize_;
    /**
     * \brief Position of the textcursor.
     */
    int cursorPos_;
    /**
     * \brief The first column in the console.
     */
    int completeStart_;
    /**
     * \brief Regex to use in the autocomplete method.
     */
    QRegExp completeRe_;

    /**
     * \brief Textcursor.
     */
    QTextCursor tc_;
    /**
     * \brief Thread to execute string.
     */
    UCQExecuteLuaThread * executeLuaThread_;
    /**
     * \brief Is true when currently a string is executed.
     */
    bool executing_;
    /**
     * \brief Is the cursor position when the execution starts.
     */
    int executionStart_;
    /**
     * \brief Is the cursor position when the execution ends.
     */
    int executionEnd_;
    /**
     * \brief Timer for displaying busy dots.
     */
    QTimer *timer;

public slots:
    void clearConsole ();
    void error ( QString );

    void executionInit  ();
    void executionReady ();
    void setFont( const QFont& );
    void showSettings();
    void stopExec();
protected slots:
    void printSelection ( QList<Letter *> selection );
    void printLetter ( Letter * );
    void executionRuns ();

signals:
    /**
     * \brief Signal to get a list of defined functions from lua.
     *
     * \param prefix Common prefix of functions to search.
     * \param list Contains the function names after the signal was
     *        processed.
     * \see LuaController::getCompletionList()
     */
    void getCompletionList ( const QString& prefix, QStringList& list);
    /**
     * \brief Signal to stop the execution from the script.
     *
     * \see LuaController::stop()
     */
    void stopExecution ();
};

#endif /* UCQCONSOLE_H_ */
